//----------------------------------------------------------------------------
// Copyright (c) 1997 by Joseph Parrello. All Rights Reserved.
// Site   : http://space.tin.it/computer/giparrel
// E-Mail : joparrel@tin.it, giusparr@tin.it, giparre@tin.it
//
// Revision:   1.1  (December 26,1997).
//
// Definition of JPMenuGadget class.
//
// Read README.TXT for use and license.
//
// Only for OWL 5.02 users.
// OWL and ObjectWindows are trademarks of Borland International.
//
// THIS CLASS AND INFORMATION IS PROVIDED "AS IS" WITHOUT WARRANTY OF
// ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO
// THE IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A
// PARTICULAR PURPOSE.
//----------------------------------------------------------------------------

#include <owl/pch.h>

#include "jpmenuga.h"

const int DefaultTextLen = 512;

//{{JPMenuGadget Implementation}}

JPMenuGadget::JPMenuGadget(TMenu& menu, TWindow* window, int id, TBorderStyle borderstyle, const char far* text, TAlign align, uint numChars, TFont* font)
:
    TTextGadget(id, borderstyle, align, numChars, text, font),
    MenuTarget(window)
{
  // INSERT>> Your constructor code here.
  SetShrinkWrap(true, true);
  GadgetPopupMenu = new TPopupMenu(menu);
  DefaultBorderStyle = borderstyle;

  char menuText[DefaultTextLen];
  if (!text)
    {
    if (GadgetPopupMenu->GetMenuString(0, menuText, DefaultTextLen, MF_BYPOSITION));
       SetText(menuText);
    }
}


JPMenuGadget::~JPMenuGadget()
{
  // INSERT>> Your destructor code here.
  delete GadgetPopupMenu;
}


void JPMenuGadget::LButtonDown(uint modKeys, TPoint& point)
{
  TTextGadget::LButtonDown(modKeys, point);

  // INSERT>> Your code here.
   TWindow* parent= Window->GetParentO();
   TDecoratedFrame* frame= parent ? TYPESAFE_DOWNCAST(parent, TDecoratedFrame) : 0;
   while (parent && !frame)
   {
     parent = parent->GetParentO();
     if (parent)
       frame = TYPESAFE_DOWNCAST(parent, TDecoratedFrame);
   }
   if (frame)
   	ParentWnd = *frame;
   else ParentWnd = *MenuTarget;
   if (ParentWnd)
     {
      TTextGadget::MouseLeave(0,point);
      TRect rect = GetBounds();
      TBorders bord = GetBorders();
      rect.top = rect.top - bord.Top;
      rect.left = rect.left - bord.Left;
      rect.bottom = rect.bottom - bord.Bottom;
      rect.right = rect.right - bord.Right;
      TPoint p2(rect.TopLeft());
      Window->ClientToScreen(p2);
      rect.right = p2.x + (rect.right - rect.left);
      rect.bottom = p2.y + (rect.bottom - rect.top);
      rect.left = p2.x;
      rect.top = p2.y;
      p2.y = rect.bottom;
      if (DefaultBorderStyle==ButtonUp)
         {
         TGadget::SetBorderStyle(ButtonDn);
         TGadget::Invalidate();
         TGadget::Update();
         }
      #if defined(BI_PLAT_WIN16)
      GadgetPopupMenu->TrackPopupMenu(TPM_LEFTALIGN | TPM_LEFTBUTTON, p2, 0, ParentWnd);
      #elif defined(BI_PLAT_WIN32)
      TrackPopupMenuEx(TPM_VERTICAL, p2, ParentWnd, &rect);
      /*GadgetPopupMenu->TrackPopupMenuEx(TPM_VERTICAL, p2, ParentWnd, &rect);*/
      #endif
      if (GetBorderStyle()!=DefaultBorderStyle)
         {
         TGadget::SetBorderStyle(DefaultBorderStyle);
         TGadget::Invalidate();
         TGadget::Update();
         }
      }
}


void JPMenuGadget::MouseEnter(uint modKeys, TPoint& point)
{
  TTextGadget::MouseEnter(modKeys, point);

  // INSERT>> Your code here.
  if (Window->GetHintMode() == TGadgetWindow::EnterHints)
        Window->SetHintCommand(GetId());
}


void JPMenuGadget::MouseLeave(uint modKeys, TPoint& point)
{
  TTextGadget::MouseLeave(modKeys, point);

  // INSERT>> Your code here.
  if (Window->GetHintMode() == TGadgetWindow::EnterHints)
        Window->SetHintCommand(-1);
}

#if defined(BI_PLAT_WIN32)
bool JPMenuGadget::TrackPopupMenuEx(uint flags, const TPoint& point,
                                        HWND wnd, const TRect* rect)
{
  TPMPARAMS tpm;

  PRECONDITION(GadgetPopupMenu->GetHandle());
  tpm.cbSize=sizeof(TPMPARAMS);
  tpm.rcExclude=*rect;
  return ::TrackPopupMenuEx(GadgetPopupMenu->GetHandle(), flags, point.x, point.y, wnd, &tpm);
}
#endif

